@@ -440,7 +440,10 @@ class ChatTable:
                 order_by = filter.get("order_by")
                 direction = filter.get("direction")
 
-                if order_by and direction and getattr(Chat, order_by):
+                if order_by and direction:
+                    if not getattr(Chat, order_by, None):
+                        raise ValueError("Invalid order_by field")
+
                     if direction.lower() == "asc":
                         query = query.order_by(getattr(Chat, order_by).asc())
                     elif direction.lower() == "desc":
@@ -762,15 +765,20 @@ class ChatTable:
                     )
 
             elif dialect_name == "postgresql":
-                # PostgreSQL relies on proper JSON query for search
+                # PostgreSQL doesn't allow null bytes in text. We filter those out by checking
+                # the JSON representation for \u0000 before attempting text extraction
                 postgres_content_sql = (
                     "EXISTS ("
                     "    SELECT 1 "
                     "    FROM json_array_elements(Chat.chat->'messages') AS message "
-                    "    WHERE LOWER(message->>'content') LIKE '%' || :content_key || '%'"
+                    "    WHERE message->'content' IS NOT NULL "
+                    "    AND (message->'content')::text NOT LIKE '%\\u0000%' "
+                    "    AND LOWER(message->>'content') LIKE '%' || :content_key || '%'"
                     ")"
                 )
                 postgres_content_clause = text(postgres_content_sql)
+                # Also filter out chats with null bytes in title
+                query = query.filter(text("Chat.title::text NOT LIKE '%\\x00%'"))
                 query = query.filter(
                     or_(
                         Chat.title.ilike(bindparam("title_key")),
